/*
 BiPase data transfer simulation

 Kirk Weedman

 May 18, 2018

*/

`timescale 1ns/100ps

module BiPhase_tb();
   parameter    OSW = 1;
   parameter    OSL = 32;
   
   reg                  clk, xreq, xrcv_ack;
   wire                 xack, xrcv_rdy;
   wire                 irst, xrst, iclk, xclk, xrdy;
   reg    [OSL*OSW-1:0] xdata;
   wire   [OSL*OSW-1:0] xrcv_data;
   wire       [OSW-1:0] din;
   wire       [OSW-1:0] dout;
   reg                  reset;

   assign xrst = reset;
   assign irst = reset;
   assign xclk = clk;
   assign iclk = clk;

   initial
   begin
  $display ("reset high");
      reset = 1'b1;
      #50;
      xreq = 1'b0;
      xrcv_ack = 1'b0;
      #500;
  $display ("reset low");
      reset = 1'b0;
      SEND_DATA(32'h9c18335A); // width should be OSL*OSW
      SEND_DATA(32'hFD3E0123); // data gets split into OSW streams, each of length OSL
      SEND_DATA(32'hF1010101);
      SEND_DATA(32'hF8888888);
      SEND_DATA(32'h2387fc87);
      SEND_DATA(32'hAA5533CC);
      SEND_DATA(32'h00FF00FF);
      $display("Successful completion. All transfers passed");
      #1000;
      $stop;
   end
   assign din = dout; // feed data from xmit to rcv
   
   // Total number of bits to transmit
   // -------------------------------------------------------------------------------------+
   //   (P.S. Change OSW = 4 and OSL = 8 and see what happens)                             |
   // ---------------------------------------------------------------------------+         |
   // Number of data streams                                                     |         |
   //                                                                            v         v
   BiPhase_xmit #(.XCLK_FREQ(122880000), .ICLK_FREQ(122880000), .DLY_TIME(1), .OSW(OSW), .OSL(OSL))
      xmit (irst, iclk, xrst, xclk, xdata, xreq, xrdy, xack, dout);

   BiPhase_rcv #(.XCLK_FREQ(122880000), .ICLK_FREQ(122880000), .OSW(OSW), .OSL(OSL))
      rcv (irst, iclk, xrst, xclk, xrcv_data, xrcv_rdy, xrcv_ack, din);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
task SEND_DATA;
input [OSL*OSW-1:0] data;
begin
   xrcv_ack = 1'b0;
   #40;
   xdata = data;  // test pattern 1
   wait (xrdy == 1'b1);
   #5
   xreq = 1'b1;
   wait (xack == 1'b1);
   #4
   xreq = 1'b0;
   wait (xrcv_rdy == 1'b1);
   if (xrcv_data !== data)
   begin
      $display("Transfer failed!");
      #1000;
      $stop;
   end
   @ (posedge xclk);
   @ (posedge xclk);   
   xrcv_ack = 1'b1; // get the data out so the next one can come in.
   #50;
end
endtask

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
localparam CLK_SPEED = 4.07;  // IF clk = 122.88Mhz = 8.14nS, For half of clock, CLK_SPEED = 8.14nS/2 = 4.07nS

initial
begin
  $display ("clk started");
  clk = 1'b0;
  #15
  forever
  begin
    #CLK_SPEED clk = 1'b1;
    #CLK_SPEED clk = 1'b0;
  end
end
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

endmodule